﻿// hardcodet.net DebugActivity for Windows Workflow
// Copyright (c) 2008 Philipp Sumi, Evolve Software Technologies
// Contact and Information: http://www.hardcodet.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the Code Project Open License (CPOL);
// either version 1.0 of the License, or (at your option) any later
// version.
// 
// This software is provided "AS IS" with no warranties of any kind.
// The entire risk arising out of the use or performance of the software
// and source code is with you.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.


using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Compiler;
using Hardcodet.WF.Util.Activities.DesignTime.Designers;
using Hardcodet.WF.Util.Activities.DesignTime.Validation;

namespace Hardcodet.WF.Util.Activities
{
  [ToolboxBitmap(typeof (DebugActivity), "DesignTime.Images.Console.png")]
  [Designer(typeof (DebugActivityDesigner))]
  [ActivityValidator(typeof (DebugActivityValidator))]
  public partial class DebugActivity : Activity
  {
    #region OutputMode dependency property

    /// <summary>
    /// Whether to perform Console.WriteLine() or Debug.WriteLine().
    /// </summary>
    public static readonly DependencyProperty OutputModeProperty;


    /// <summary>
    /// A property wrapper for the <see cref="OutputModeProperty"/>
    /// dependency property:<br/>
    /// Whether to perform Console.WriteLine() or Debug.WriteLine().
    /// </summary>
    [Description("Whether to perform Console.WriteLine() or Debug.WriteLine().")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(OutputMode.Console)]
    public OutputMode OutputMode
    {
      get { return (OutputMode) GetValue(OutputModeProperty); }
      set { SetValue(OutputModeProperty, value); }
    }

    #endregion

    #region OutputMask dependency property

    /// <summary>
    /// The string that should be written to the console.
    /// </summary>
    public static readonly DependencyProperty OutputMaskProperty;


    /// <summary>
    /// A property wrapper for the <see cref="OutputMaskProperty"/>
    /// dependency property:<br/>
    /// The string that should be written to the console.
    /// </summary>
    [Description("The string (or string mask) that should be written to the console.")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue("")]
    [Editor(typeof(MultilineStringEditor), typeof(UITypeEditor))]
    public string OutputMask
    {
      get { return (string) GetValue(OutputMaskProperty); }
      set { SetValue(OutputMaskProperty, value); }
    }

    #endregion

    #region OutputParameter0 dependency property

    /// <summary>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    public static readonly DependencyProperty OutputParameter0Property;


    /// <summary>
    /// A property wrapper for the <see cref="OutputParameter0Property"/>
    /// dependency property:<br/>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    [Description("An optional parameter for the output mask ({0}).")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(null)]
    public object OutputParameter0
    {
      get { return (object) GetValue(OutputParameter0Property); }
      set { SetValue(OutputParameter0Property, value); }
    }

    #endregion

    #region OutputParameter1 dependency property

    /// <summary>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    public static readonly DependencyProperty OutputParameter1Property;


    /// <summary>
    /// A property wrapper for the <see cref="OutputParameter1Property"/>
    /// dependency property:<br/>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    [Description("An optional parameter for the output mask ({1}).")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(null)]
    public object OutputParameter1
    {
      get { return (object) GetValue(OutputParameter1Property); }
      set { SetValue(OutputParameter1Property, value); }
    }

    #endregion

    #region OutputParameter2 dependency property

    /// <summary>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    public static readonly DependencyProperty OutputParameter2Property;


    /// <summary>
    /// A property wrapper for the <see cref="OutputParameter2Property"/>
    /// dependency property:<br/>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    [Description("An optional parameter for the output mask ({2}).")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(null)]
    public object OutputParameter2
    {
      get { return (object) GetValue(OutputParameter2Property); }
      set { SetValue(OutputParameter2Property, value); }
    }

    #endregion

    #region OutputParameter3 dependency property

    /// <summary>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    public static readonly DependencyProperty OutputParameter3Property;


    /// <summary>
    /// A property wrapper for the <see cref="OutputParameter3Property"/>
    /// dependency property:<br/>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    [Description("An optional parameter for the output mask ({3}).")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(null)]
    public object OutputParameter3
    {
      get { return (object) GetValue(OutputParameter3Property); }
      set { SetValue(OutputParameter3Property, value); }
    }

    #endregion

    #region OutputParameter4 dependency property

    /// <summary>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    public static readonly DependencyProperty OutputParameter4Property;


    /// <summary>
    /// A property wrapper for the <see cref="OutputParameter4Property"/>
    /// dependency property:<br/>
    /// An optional parameter that can be refered to in the output mask.
    /// </summary>
    [Description("An optional parameter for the output mask ({4}).")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(null)]
    public object OutputParameter4
    {
      get { return (object) GetValue(OutputParameter4Property); }
      set { SetValue(OutputParameter4Property, value); }
    }

    #endregion

    #region SpaceBefore dependency property

    /// <summary>
    /// Number of empty lines preceeding the output.
    /// </summary>
    public static readonly DependencyProperty SpaceBeforeProperty;


    /// <summary>
    /// A property wrapper for the <see cref="SpaceBeforeProperty"/>
    /// dependency property:<br/>
    /// Number of empty lines preceeding the output.
    /// </summary>
    [Description("Number of empty lines preceeding the output.")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(0)]
    public int SpaceBefore
    {
      get { return (int) GetValue(SpaceBeforeProperty); }
      set { SetValue(SpaceBeforeProperty, Math.Max(0, value)); }
    }

    #endregion

    #region SpaceAfter dependency property

    /// <summary>
    /// Number of empty lines following the output.
    /// </summary>
    public static readonly DependencyProperty SpaceAfterProperty;


    /// <summary>
    /// A property wrapper for the <see cref="SpaceAfterProperty"/>
    /// dependency property:<br/>
    /// Number of empty lines following the output.
    /// </summary>
    [Description("Number of empty lines following the output.")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(0)]
    public int SpaceAfter
    {
      get { return (int) GetValue(SpaceAfterProperty); }
      set { SetValue(SpaceAfterProperty, Math.Max(0, value)); }
    }

    #endregion

    #region Condition dependency property

    /// <summary>
    /// A condition that is being evaluated before writing to the selected output.
    /// </summary>
    public static readonly DependencyProperty ConditionProperty;


    /// <summary>
    /// A property wrapper for the <see cref="ConditionProperty"/>
    /// dependency property:<br/>
    /// A condition that is being evaluated before writing to the selected output.
    /// </summary>
    [Description("A condition that is filters output. Must evaluate to false for Debug.Assert, to true for other output.")]
    [Category("Output")]
    [Browsable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [DefaultValue(null)]
    public ActivityCondition Condition
    {
      get { return (ActivityCondition) GetValue(ConditionProperty); }
      set { SetValue(ConditionProperty, value); }
    }

    #endregion


    #region construction

    /// <summary>
    /// Registers dependency properties.
    /// </summary>
    static DebugActivity()
    {
      //register dependency property
      PropertyMetadata md = new PropertyMetadata(OutputMode.Console);
      OutputModeProperty = DependencyProperty.Register("OutputMode", typeof (OutputMode), typeof (DebugActivity),
                                                       md);

      md = new PropertyMetadata("");
      OutputMaskProperty = DependencyProperty.Register("OutputMask", typeof (string),
                                                       typeof (DebugActivity), md);


      md = new PropertyMetadata(null);
      OutputParameter0Property = DependencyProperty.Register("OutputParameter0", typeof (object),
                                                             typeof (DebugActivity), md);

      md = new PropertyMetadata(null);
      OutputParameter1Property = DependencyProperty.Register("OutputParameter1", typeof (object),
                                                             typeof (DebugActivity), md);

      md = new PropertyMetadata(null);
      OutputParameter2Property = DependencyProperty.Register("OutputParameter2", typeof (object),
                                                             typeof (DebugActivity), md);

      md = new PropertyMetadata(null);
      OutputParameter3Property = DependencyProperty.Register("OutputParameter3", typeof (object),
                                                             typeof (DebugActivity), md);

      md = new PropertyMetadata(null);
      OutputParameter4Property = DependencyProperty.Register("OutputParameter4", typeof (object),
                                                             typeof (DebugActivity), md);

      md = new PropertyMetadata(0);
      SpaceBeforeProperty = DependencyProperty.Register("SpaceBefore", typeof (int), typeof (DebugActivity), md);

      md = new PropertyMetadata(0);
      SpaceAfterProperty = DependencyProperty.Register("SpaceAfter", typeof (int), typeof (DebugActivity), md);

      md = new PropertyMetadata(null);
      ConditionProperty = DependencyProperty.Register("Condition", typeof (ActivityCondition), typeof (DebugActivity), md);                                                      
    }


    public DebugActivity()
    {
      InitializeComponent();
    }

    #endregion


    /// <summary>
    /// Writes the configured string (with optional parameters) to console or
    /// debug output.
    /// </summary>
    /// <param name="executionContext"></param>
    /// <returns></returns>
    protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
    {
      //evaluate condition
      bool conditionResult = Condition == null || Condition.Evaluate(this, executionContext);

      //if we have an invalid condition and we are *not* supposed to perform an assertion, break here
      if (!conditionResult && OutputMode != OutputMode.Assert) return base.Execute(executionContext);

      //generate prefix / suffix
      string prefix = "";
      for (int i = 0; i < SpaceBefore; i++) prefix += Environment.NewLine;
      string suffix = "";
      for (int i = 0; i < SpaceAfter; i++) suffix += Environment.NewLine;

      //create argument array
      object[] data = {OutputParameter0, OutputParameter1, OutputParameter2};
      string output = String.Format("{0}{1}{2}", prefix, OutputMask, suffix);
      try
      {
        output = String.Format(output, data);
      }
      catch (FormatException)
      {
        //generate alternative output
        output = "\n*****\nConsole output failed on {0} {1}, probably due to invalid parameter indices.\n*****\n";
        output = String.Format(output, GetType().Name, Name);
      }


      //write to selected output
      switch (OutputMode)
      {
        case OutputMode.Console:
          Console.Out.WriteLine(output);
          break;
        case OutputMode.Debug:
          Debug.WriteLine(output);
          break;
        case OutputMode.Assert:
          Debug.Assert(conditionResult, output);
          break;
      }

      return base.Execute(executionContext);
    }
  }
}