﻿// hardcodet.net DebugActivity for Windows Workflow
// Copyright (c) 2008 Philipp Sumi, Evolve Software Technologies
// Contact and Information: http://www.hardcodet.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the Code Project Open License (CPOL);
// either version 1.0 of the License, or (at your option) any later
// version.
// 
// This software is provided "AS IS" with no warranties of any kind.
// The entire risk arising out of the use or performance of the software
// and source code is with you.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.$


using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using Hardcodet.WF.Util.Activities.DesignTime.Themes;
using Hardcodet.WF.Util.Properties;

namespace Hardcodet.WF.Util.Activities.DesignTime.Designers
{
  /// <summary>
  /// A custom designer for the <see cref="DebugActivity"/>.
  /// </summary>
  [ActivityDesignerTheme(typeof (DarkActivityTheme))]
  internal sealed class DebugActivityDesigner : ActivityDesigner
  {
    protected override void Initialize(Activity activity)
    {
      base.Initialize(activity);
      UpdateDesigner((DebugActivity) activity);
    }

    protected override void OnActivityChanged(ActivityChangedEventArgs e)
    {
      UpdateDesigner((DebugActivity) e.Activity);
      base.OnActivityChanged(e);
    }


    /// <summary>
    /// Updates the designer surface in order to reflect the 
    /// applied <see cref="DebugActivity.OutputMode"/>
    /// property.
    /// </summary>
    /// <param name="activity">The processed activity.</param>
    private void UpdateDesigner(DebugActivity activity)
    {
      switch (activity.OutputMode)
      {
        case OutputMode.Console:
          Text = "Console Output";
          Image = Resources.Console;
          break;
        case OutputMode.Debug:
          Text = "Debug Output";
          Image = Resources.Debug;
          break;
        case OutputMode.Assert:
          Text = "Debug Assertion";
          Image = Resources.Assert;
          break;
      }
    }
  }
}