﻿// hardcodet.net DebugActivity for Windows Workflow
// Copyright (c) 2008 Philipp Sumi, Evolve Software Technologies
// Contact and Information: http://www.hardcodet.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the Code Project Open License (CPOL);
// either version 1.0 of the License, or (at your option) any later
// version.
// 
// This software is provided "AS IS" with no warranties of any kind.
// The entire risk arising out of the use or performance of the software
// and source code is with you.
//
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.


using System;
using System.Workflow.ComponentModel.Compiler;

namespace Hardcodet.WF.Util.Activities.DesignTime.Validation
{
  /// <summary>
  /// Makes sure an output string has been defined for a given
  /// <see cref="DebugActivity"/>.
  /// </summary>
  public class DebugActivityValidator : ActivityValidator
  {
    /// <summary>
    /// Performs validation of output mask and assertion parameters.
    /// </summary>
    /// <param name="manager"></param>
    /// <param name="obj"></param>
    /// <returns></returns>
    public override ValidationErrorCollection Validate(ValidationManager manager, object obj)
    {
      //perform basic validation
      ValidationErrorCollection errors = base.Validate(manager, obj);

      //only perform validation if the activity is part of a workflow
      DebugActivity activity = obj as DebugActivity;
      if (activity == null || activity.Parent == null)
      {
        return errors;
      }

      //make sure the output mask has been defined explicitly or through a binding expression
      if (!activity.IsBindingSet(DebugActivity.OutputMaskProperty) && String.IsNullOrEmpty(activity.OutputMask))
      {
        errors.Add(ValidationError.GetNotSetValidationError("OutputMask"));
      }

      //in case we have assertion output, make sure the condition is set
      if (activity.OutputMode == OutputMode.Assert && activity.Condition == null &&
          !activity.IsBindingSet(DebugActivity.ConditionProperty))
      {
        errors.Add(ValidationError.GetNotSetValidationError("Condition"));
      }

      return errors;
    }
  }
}