﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Workflow.Runtime;

namespace Hardcodet.WF.Util.Hosting
{
  /// <summary>
  /// Wraps a single workflow instance, along with state-related data
  /// (e.g. output parameters, exception information).
  /// </summary>
  public class WorkflowInstanceWrapper
  {
    #region properties

    /// <summary>
    /// The workflow instance to be used.
    /// </summary>
    public WorkflowInstance WorkflowInstance { get; private set; }

    /// <summary>
    /// Gets or sets the output parameters of the managed workflow
    /// instance.
    /// </summary>
    public Dictionary<string, object> OutputParameters { get; set; }

    /// <summary>
    /// A wait handle that the host applicatino can use
    /// if it wants to halt processing until the workflow
    /// has completed.
    /// </summary>
    public ManualResetEvent WaitHandle { get; set; }

    /// <summary>
    /// An exception if one was thrown from the workflow.
    /// </summary>
    public Exception Exception { get; set; }

    /// <summary>
    /// Indicates why the managed workflow was suspended. Defaults to
    /// <see cref="String.Empty"/>
    /// </summary>
    public string ReasonSuspended { get; set; }

    #endregion

    /// <summary>
    /// Inits the wrapper class with the workflow instance to be maintained.
    /// </summary>
    /// <param name="instance">The workflow instance to be maintained.</param>
    /// <exception cref="ArgumentNullException">If <paramref name="instance"/>
    /// is a null reference.</exception>
    public WorkflowInstanceWrapper(WorkflowInstance instance)
    {
      if (instance == null) throw new ArgumentNullException("instance");
      WorkflowInstance = instance;

      //init auto properties with default values
      WaitHandle = new ManualResetEvent(false);
      OutputParameters = new Dictionary<string, object>();
      ReasonSuspended = String.Empty;
    }


    /// <summary>
    /// Signal that the workflow has finished and the host application
    /// may stop waiting for the workflow to complete.
    /// </summary>
    public void StopWaiting()
    {
      WaitHandle.Set();
    }
  }
}