﻿using System;

namespace Hardcodet.WF.Util.Hosting
{
  /// <summary>
  /// Encapsulates information about a workflow related event
  /// that is reported by a <see cref="WorkflowRuntimeManager"/>.
  /// </summary>
  public class WorkflowLogEventArgs : EventArgs
  {
    /// <summary>
    /// The underlying workflow that caused the event.
    /// </summary>
    public WorkflowInstanceWrapper Workflow { get; private set; }

    /// <summary>
    /// Indicates the state of the workflow when the event
    /// occurred.
    /// </summary>
    public WorkflowState State { get; private set; }

    /// <summary>
    /// The reported message.
    /// </summary>
    public string Message { get; private set; }


    /// <summary>
    /// 
    /// </summary>
    /// <param name="workflow"></param>
    /// <param name="state"></param>
    /// <param name="message"></param>
    /// <exception cref="ArgumentNullException">If <paramref name="workflow"/>
    /// is a null reference.</exception>
    public WorkflowLogEventArgs(WorkflowInstanceWrapper workflow, WorkflowState state, string message)
    {
      if (workflow == null) throw new ArgumentNullException("workflow");

      Workflow = workflow;
      State = state;
      Message = message;
    }
  }
}